==> kartofel-1.1.tar.gz: kartofel-1.1/saved_game.cc <==
//  This file is part of the computer game Kartofel.
//  Copyright (C) 2008 Pawel Aleksander Fedorynski <pfedor@fuw.edu.pl>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <cassert>
#include <cstdlib>
#include <sstream>
#include <zlib.h>

#include "saved_game.h"

using namespace std;

namespace kartofel {

SavedGame::SavedGame(int starting_level)
  : starting_level_(starting_level) { }

void SavedGame::Reset(int starting_level)
{
  starting_level_ = starting_level;
  history_.clear();
}

// Returns a number between 0 and 31.
int EncodeInput(const Input& input)
{
  int retv = 0;
  if (input.left) {
    retv += 1;
  }
  if (input.right) {
    retv += 2;
  }
  if (input.accelerate) {
    retv += 4;
  }
  if (input.deccelerate) {
    retv += 8;
  }
  if (input.start) {
    retv += 16;
  }
  return retv;
}

bool DecodeInput(int enc, Input* input)
{
  if (enc < 0 || enc > 31) {
    return false;
  }
  input->left = (enc & 1);
  input->right = (enc & 2);
  input->accelerate = (enc & 4);
  input->deccelerate = (enc & 8);
  input->start = (enc & 16);

  return true;
}

void SavedGame::AddStep(const Input& input)
{
  history_.push_back(EncodeInput(input));
}

void SavedGame::Encode(string* out) const
{
  stringstream ss;
  ss << starting_level_ << " ";
  string uncompressed = ss.str();
  for (int i = 0; i < history_.size(); i++) {
    char c = history_[i];
    uncompressed.append(1, c);
  }
  uLong compr_len = DoubleToInt(uncompressed.size() * 1.001) + 13;
  Byte* compr = new Byte[compr_len];
  int rc = compress(compr, &compr_len,
                    (const Bytef*)uncompressed.data(),
                    uncompressed.size());
  if (rc != Z_OK) {
    cerr << "compress() failed; error code: " << rc << endl;
    exit(1);
  }
  ss.str("");
  ss << uncompressed.size() << " ";
  out->assign(ss.str());
  out->append((char*)compr, compr_len);
  delete [] compr;
}

bool SavedGame::Decode(const string& compressed)
{
  int n = compressed.find(" ");
  if (n == string::npos) {
    return false;
  }
  string encoded_len_str(compressed.begin(), compressed.begin() + n);
  size_t uncompr_len;
  if (1 != sscanf(encoded_len_str.c_str(), "%zu", &uncompr_len)) {
    return false;
  }
  Byte* uncompr = new Byte[uncompr_len];
  uLong uncompr_len_ul = uncompr_len;
  int rc = uncompress(uncompr, &uncompr_len_ul,
                      (const Bytef*)(compressed.data() + n + 1),
                      compressed.size() - n - 1);
  string encoded((char*)uncompr, uncompr_len_ul);
  delete [] uncompr;
  if (rc != Z_OK) {
    cerr << "uncompress() failed; error code: " << rc << endl;
    return false;
  }
  history_.clear();
  n = encoded.find(" ");
  if (n == string::npos) {
    return false;
  }
  string starting_level_str(encoded.begin(), encoded.begin() + n);
  if (1 != sscanf(starting_level_str.c_str(), "%d", &starting_level_)) {
    return false;
  }
  if (starting_level_ < 1) {
    return false;
  }
  for (int i = n + 1; i < encoded.size(); i++) {
    Input input;
    if (!DecodeInput(encoded[i], &input)) {
      return false;    
    }
    history_.push_back(encoded[i]);
  }
  return true;
}

bool SavedGame::Verify(const vector<Level>& levels, int* score) const
{
  if (levels.size() < starting_level_) {
    return false;  
  }
  Game game;
  int level = starting_level_;
  game.SetLevel(levels[level - 1], level);
  for (int i = 0; i < history_.size(); i++) {
    vector<Action> actions;
    int timestamp = game.GetCurrentTimestamp() + STEP_TS;
    Input input;
    bool ok = DecodeInput(history_[i], &input);
    assert(ok);
    game.Advance(input, timestamp, &actions);
    for (int j = 0; j < actions.size(); j++) {      
      if (actions[j].type == EXPLOSION || actions[j].type == OUT_OF_TIME) {
        if (game.GetLives() == 0) {
          if (i == history_.size() - 1) {
            *score = game.GetScore();
            return true;
          } else {
            return false;
          }
        }
        game.SetLevel(levels[level - 1], level);
      } else if (actions[j].type == END_LEVEL) {
        int min_speed = game.GetMinimumSpeedUsed();
        int time_left = game.GetTimeLeft();
        int tlms = time_left * STEP_MILLISECONDS / STEP_TS;
        int tlds = tlms / 100;
        int bonus = tlds * min_speed * min_speed; 
        game.AddScore(bonus);

        if (level == levels.size()) {
          if (i == history_.size() - 1) {
            *score = game.GetScore();
            return true;
          } else {
            return false;
          }
        }
        level++;
        game.SetLevel(levels[level - 1], level);
      }
    }
  }
  return false;
}

}  // namespace kartofel

==> kartofel-1.2.tar.gz: kartofel-1.2/saved_game.cc <==
//  This file is part of the computer game Kartofel.
//  Copyright (C) 2008 Pawel Aleksander Fedorynski <pfedor@fuw.edu.pl>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <cassert>
#include <cstdlib>
#include <sstream>
#include <zlib.h>

#include "saved_game.h"

using namespace std;

namespace kartofel {

SavedGame::SavedGame(int starting_level)
  : starting_level_(starting_level) { }

void SavedGame::Reset(int starting_level)
{
  starting_level_ = starting_level;
  history_.clear();
}

// Returns a number between 0 and 31.
int EncodeInput(const Input& input)
{
  int retv = 0;
  if (input.left) {
    retv += 1;
  }
  if (input.right) {
    retv += 2;
  }
  if (input.accelerate) {
    retv += 4;
  }
  if (input.deccelerate) {
    retv += 8;
  }
  if (input.start) {
    retv += 16;
  }
  return retv;
}

bool DecodeInput(int enc, Input* input)
{
  if (enc < 0 || enc > 31) {
    return false;
  }
  input->left = (enc & 1);
  input->right = (enc & 2);
  input->accelerate = (enc & 4);
  input->deccelerate = (enc & 8);
  input->start = (enc & 16);

  return true;
}

void SavedGame::AddStep(const Input& input)
{
  history_.push_back(EncodeInput(input));
}

void SavedGame::Encode(string* out) const
{
  stringstream ss;
  ss << starting_level_ << " ";
  string uncompressed = ss.str();
  for (int i = 0; i < history_.size(); i++) {
    char c = history_[i];
    uncompressed.append(1, c);
  }
  uLong compr_len = DoubleToInt(uncompressed.size() * 1.001) + 13;
  Byte* compr = new Byte[compr_len];
  int rc = compress(compr, &compr_len,
                    (const Bytef*)uncompressed.data(),
                    uncompressed.size());
  if (rc != Z_OK) {
    cerr << "compress() failed; error code: " << rc << endl;
    exit(1);
  }
  ss.str("");
  ss << uncompressed.size() << " ";
  out->assign(ss.str());
  out->append((char*)compr, compr_len);
  delete [] compr;
}

bool SavedGame::Decode(const string& compressed)
{
  int n = compressed.find(" ");
  if (n == string::npos) {
    return false;
  }
  string encoded_len_str(compressed.begin(), compressed.begin() + n);
  size_t uncompr_len;
  if (1 != sscanf(encoded_len_str.c_str(), "%zu", &uncompr_len)) {
    return false;
  }
  Byte* uncompr = new Byte[uncompr_len];
  uLong uncompr_len_ul = uncompr_len;
  int rc = uncompress(uncompr, &uncompr_len_ul,
                      (const Bytef*)(compressed.data() + n + 1),
                      compressed.size() - n - 1);
  string encoded((char*)uncompr, uncompr_len_ul);
  delete [] uncompr;
  if (rc != Z_OK) {
    cerr << "uncompress() failed; error code: " << rc << endl;
    return false;
  }
  history_.clear();
  n = encoded.find(" ");
  if (n == string::npos) {
    return false;
  }
  string starting_level_str(encoded.begin(), encoded.begin() + n);
  if (1 != sscanf(starting_level_str.c_str(), "%d", &starting_level_)) {
    return false;
  }
  if (starting_level_ < 1) {
    return false;
  }
  for (int i = n + 1; i < encoded.size(); i++) {
    Input input;
    if (!DecodeInput(encoded[i], &input)) {
      return false;    
    }
    history_.push_back(encoded[i]);
  }
  return true;
}

bool SavedGame::Verify(const vector<Level>& levels,
                       int* starting_level, int* end_level, int* score) const
{
  if (levels.size() < starting_level_) {
    //cout << "1\n" << endl;
    return false;  
  }
  Game game;
  int level = starting_level_;
  //cout << "level: " << level << endl;
  *starting_level = level;
  game.SetLevel(levels[level - 1], level);
  for (int i = 0; i < history_.size(); i++) {
    vector<Action> actions;
    int timestamp = game.GetCurrentTimestamp() + STEP_TS;
    Input input;
    bool ok = DecodeInput(history_[i], &input);
    assert(ok);
    cout << "timestamp: " << timestamp << endl;
    DebugPrintInput(input);
    game.Advance(input, timestamp, &actions);
    for (int j = 0; j < actions.size(); j++) {
      DebugPrintAction(actions[j]);
      if (actions[j].type == EXPLOSION || actions[j].type == OUT_OF_TIME) {
        if (game.GetLives() == 0) {
          if (i == history_.size() - 1) {
            *score = game.GetScore();
            *end_level = level;
            return true;
          } else {
            //cout << "2\n";
            return false;
          }
        }
        game.SetLevel(levels[level - 1], level);
      } else if (actions[j].type == END_LEVEL) {
        int min_speed = game.GetMinimumSpeedUsed();
        int time_left = game.GetTimeLeft();
        int tlms = time_left * STEP_MILLISECONDS / STEP_TS;
        int tlds = tlms / 100;
        int bonus = tlds * min_speed * min_speed; 
        game.AddScore(bonus);

        if (level == levels.size()) {
          if (i == history_.size() - 1) {
            *score = game.GetScore();
            *end_level = level;
            return true;
          } else {
            //cout << "3\n";
            return false;
          }
        }
        level++;
        //cout << "level: " << level << endl;
        game.SetLevel(levels[level - 1], level);
      }
    }
  }
  //cout << "4\n";
  return false;
}

}  // namespace kartofel
